﻿package  {
	//import the classes we need
	import flash.net.URLLoader;
	import flash.net.URLRequest;
	import flash.events.Event;
	import flash.display.Loader;
	import flash.display.Bitmap;
	import flash.display.Sprite;
	
	public class ImageLoader extends Sprite{
		//will load our xml
		private var xmlLoader:URLLoader = new URLLoader();
		
		//stores the xml
		private var xml:XML;
		
		//a list of file links to the images
		private var imageFileList:XMLList;
		
		//the path to the .swf file
		private var swfPath:String;
		
		//the project folder path (where the folders bin, src and lib are located)
		private var projectPath:String;
		
		//we must know the title of our project, so we can edit the .swf url to get our projectPath
		private var projectTitle:String;
		
		//we need to count how many images we've already loaded
		private var count:uint = 0;
		
		//we must store the images, the images are Bitmaps
		public var images:Vector.<Bitmap> = new Vector.<Bitmap>;
		
		public function ImageLoader(_projectTitle:String) {
			//we must know when ImageLoader has been added, then we can figure out what the root's url is
			addEventListener(Event.ADDED, whenAdded);
			
			//store the project's title
			projectTitle = _projectTitle;
			
			//when imageLoader is done with loading, call xmlLoaded
			xmlLoader.addEventListener(Event.COMPLETE, xmlLoaded);
		}
		
		private function whenAdded(event:Event):void {
			//store the swf path
			swfPath = root.loaderInfo.url;
			
			//get our projectPath
			projectPath = swfPath.substr(0, swfPath.length - projectTitle.length);
		}
		
		//this function will load the images from an xml, and store the images
		public function load(url:String):void {
			//load the xml, which we'll need to load the images
			xmlLoader.load(new URLRequest(projectPath + url));			
		}
		
		private function xmlLoaded(event:Event):void {
			//store the xml, which xmlLoader has loaded
			xml = new XML(event.target.data);
			
			//store all the file links in an XMLList
			imageFileList = new XMLList(xml.image.file);
			
			//now that we have the file information, we can load the images
			loadImages();
		}
		
		private function loadImages():void {
			for each (var imageFile:String in imageFileList){
				//create
				var imageLoader:Loader = new Loader();
				//whenever an image has been loaded, call imageLoaded
				imageLoader.contentLoaderInfo.addEventListener(Event.COMPLETE, imageLoaded, false, 0, true);
				imageLoader.load(new URLRequest(projectPath + imageFile));
			}
		}
		
		private function imageLoaded(event:Event):void {
			//we're counting how many images we've already loaded, we've just loaded (another) one
			count++;
			
			//get the image Bitmap
			var image:Bitmap = Bitmap(event.target.loader.content);
			
			//store the image in our images Vector
			images.push(image);
			
			//if this is the last image we needed to load
			if(count == imageFileList.length()){
				//we're done, dispatch an event
				dispatchEvent(new Event('imagesLoaded'));
			}
			
			//--cleanup
			//unload the loader
			event.target.loader.unload();
		}
		
		public function cleanUp():void {
			xmlLoader.data = null;
			xmlLoader = null;
			xml = null;
			imageFileList = null;
			count = 0;
			images = null;
		}
	}
	
}