﻿package {
	import flash.display.MovieClip;
	import flash.events.MouseEvent;

	import caurina.transitions.*;
	import caurina.transitions.properties.*;
	import flash.display.Bitmap;
	import flash.events.Event;

	public class Polaroid extends MovieClip {

		//the scale at which the Polaroid is zoomed in
		private static const ZOOMED_IN_SCALE:Number = 1;
		//the scale at which the Polaroid is zoomed out
		private static const ZOOMED_OUT_SCALE:Number = .6;

		//the max height an image can be
		private var IMAGE_MAX_HEIGHT:Number;
		//the max width an image can be
		private var IMAGE_MAX_WIDTH:Number;

		//we must scale the loaded images, remember the scale
		private var scale:Number;

		//when we zoom out, we zoom out back to the Polaroid's original position and rotation
		//the Polaroid's original x coordinate.
		private var origX:Number;
		//we also need the original y coordinate
		private var origY:Number;
		//store the Polaroid's original rotation
		private var origRotation:Number;
		//when this Boolean becomes true, the Polaroid will not respond to mouse events
		private var muted:Boolean;
		//we need to know if the Polaroid has been clicked
		public var clicked:Boolean;

		public function Polaroid(image:Bitmap) {
			//make the Polaroid a button
			buttonMode = true;

			//the maximum dimensions for an image are limited by the dimensions of imageHolder
			IMAGE_MAX_HEIGHT = imageHolder.height;
			IMAGE_MAX_WIDTH = imageHolder.width;

			//scale the Polaroid down
			scaleX = ZOOMED_OUT_SCALE;
			scaleY = ZOOMED_OUT_SCALE;


			//we'll need to resize the image to make it fit
			//first check if it is a portrait or landscape
			if (image.width > image.height) {
				//it is a landscape
				//the scale is limited by the image's height
				scale = IMAGE_MAX_HEIGHT / image.height;
			} else {
				//it is a portrait
				//the scale is limited by the image's width
				scale = IMAGE_MAX_WIDTH / image.width;
			}

			//scale the image
			image.scaleX = scale;
			image.scaleY = scale;

			//add the scaled image 
			imageHolder.addChild(image);

			//add event listeners to the Polaroid;
			addEventListener(MouseEvent.MOUSE_OVER, onMouseOver);
			addEventListener(MouseEvent.MOUSE_OUT, onMouseOut);
			addEventListener(MouseEvent.CLICK, onClick);
			addEventListener(Event.ADDED, whenAdded);
		}

		public function mute():void {
			//calling this function will make the Polaroid not respond to events
			buttonMode = false;
			muted = true;
		}

		public function unmute():void {
			//calling this function will make the Polaroid respond to events
			buttonMode = true;
			muted = false;
		}

		private function whenAdded(event:Event):void {
			//store the Polaroid's original coordinates and rotation
			origX = x;
			origY = y;
			origRotation = rotation;

			//we won't need this event listener anymore, remove it
			removeEventListener(Event.ADDED, whenAdded);
		}

		private function onMouseOver(event:MouseEvent):void {
			//only respond if the Polaroid has not been muted
			if (! muted) {
				//only respond if the Polaroid has not been clicked, clicked Polaroids should not respond to mouse overs.
				if (! clicked) {
					//move the polaroid to the front
					parent.setChildIndex(this, parent.numChildren - 1);
					//move and turn the polaroid in a random direction
					Tweener.addTween(this, {x: origX + Math.random() * 30 - 15, y: origY + Math.random() * 30 -15, rotation:origRotation + Math.random() * 20 - 10, time:1});
				}
			}
		}

		private function onMouseOut(event:MouseEvent):void {
			//only respond if the Polaroid has not been muted
			if (! muted) {
				//only respond if the Polaroid has not been clicked
				if (! clicked) {
					//move the polaroid back to it's original position and rotation
					Tweener.addTween(this, {x: origX, y:origY, rotation:origRotation, time:1});
				}
			}
		}

		private function onClick(event:MouseEvent):void {
			//only respond if the Polaroid has not been muted
			if (! muted) {
				//if it hasn't been clicked before
				if (! clicked) {
					//we've been clicked, let's make that true
					clicked = true;

					//we're zooming in, let's dispatch an event (Main will pick this up)
					dispatchEvent(new Event('zoomIn'));

					//zoom the Polaroid in to the center of the stage
					Tweener.addTween(this, {x:stage.stageWidth/2 - width/2, y:stage.stageHeight/2 - height/2, rotation:0, scaleX:ZOOMED_IN_SCALE, scaleY:ZOOMED_IN_SCALE, time: 1});
				} else {
					//we've already been clicked, so make clicked false
					clicked = false;

					//we're zooming out, let's dispatch an event (Main will pick this up)
					dispatchEvent(new Event('zoomOut'));

					//zoom the polaroid out, back to its original position
					Tweener.addTween(this, {x: origX, y:origY, rotation:origRotation, scaleX: ZOOMED_OUT_SCALE, scaleY:ZOOMED_OUT_SCALE, time:1});
				}
			}
		}
	}
}